/********************************************************************\
 * Mu (Miscellaneous Utilities) is a general convenience library.   *
 * Copyright (C) 2020  Asher Gordon <AsDaGo@posteo.net>             *
 *                                                                  *
 * This file is part of Mu.                                         *
 *                                                                  *
 * Mu is free software: you can redistribute it and/or modify it    *
 * under the terms of the GNU General Public License as published   *
 * by the Free Software Foundation, either version 3 of the         *
 * License, or (at your option) any later version.                  *
 *                                                                  *
 * Mu is distributed in the hope that it will be useful, but        *
 * WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    *
 * GNU General Public License for more details.                     *
 *                                                                  *
 * You should have received a copy of the GNU General Public        *
 * License along with Mu.  If not, see                              *
 * <https://www.gnu.org/licenses/>.                                 *
\********************************************************************/

/*
 * safe.h -- versions of standard functions that do not return on
 * error
 */

#ifndef _MU_SAFE_H
#define _MU_SAFE_H

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>

#define _MU_SAFE_TR_NAME(name) _##name##_line
#define _MU_SAFE_DECL(name, ...)					\
  _MU_SAFE_TR_NAME(name)						\
    (const char *file, int line, const char *func, __VA_ARGS__)
#define _MU_SAFE_DEF(name, ...)						\
  _MU_SAFE_TR_NAME(name)(__FILE__, __LINE__, __func__, __VA_ARGS__)

#define mu_xmalloc(size)	_MU_SAFE_DEF(mu_xmalloc, size)
#define mu_xcalloc(nmemb, size)	_MU_SAFE_DEF(mu_xcalloc, nmemb, size)
#define mu_xrealloc(ptr, size)	_MU_SAFE_DEF(mu_xrealloc, ptr, size)
#define mu_xreallocarray(ptr, nmemb, size)		\
  _MU_SAFE_DEF(mu_xreallocarray, ptr, nmemb, size)
#define mu_xstrdup(str)		_MU_SAFE_DEF(mu_xstrdup, str)
/* Format is part of the '...' so there is always at least one
   __VA_ARGS__ for a valid call of mu_xasprintf(). */
#define mu_xasprintf(strp, ...)	_MU_SAFE_DEF(mu_xasprintf, strp, __VA_ARGS__)
#define mu_xvasprintf(strp, fmt, ap)		\
  _MU_SAFE_DEF(mu_xvasprintf, strp, fmt, ap)
#define mu_xformat(stream, width, col, indent, subindent, ...)		\
  _MU_SAFE_DEF(mu_xformat, stream, width, col, indent, subindent, __VA_ARGS__)
#define mu_xformat_string(width, col, indent, subindent, ...)		\
  _MU_SAFE_DEF(mu_xformat, width, col, indent, subindent, __VA_ARGS__)
#define mu_xvformat(stream, width, col, indent, subindent, fmt, ap)	\
  _MU_SAFE_DEF(mu_xvformat, stream, width, col, indent, subindent, fmt, ap)
#define mu_xvformat_string(width, col, indent, subindent, fmt, ap)	\
  _MU_SAFE_DEF(mu_xvformat_string, width, col, indent, subindent, fmt, ap)

void *_MU_SAFE_DECL(mu_xmalloc, size_t);
void *_MU_SAFE_DECL(mu_xcalloc, size_t, size_t);
void *_MU_SAFE_DECL(mu_xrealloc, void *, size_t);
void *_MU_SAFE_DECL(mu_xreallocarray, void *, size_t, size_t);
char *_MU_SAFE_DECL(mu_xstrdup, const char *);
unsigned _MU_SAFE_DECL(mu_xasprintf, char **, const char *, ...);
unsigned _MU_SAFE_DECL(mu_xvasprintf, char **, const char *, va_list);
void _MU_SAFE_DECL(mu_xformat,
		   FILE *, unsigned short, unsigned short, unsigned short *,
		   unsigned short, unsigned short, const char *, ...);
char *_MU_SAFE_DECL(mu_xformat_string,
		    unsigned short, unsigned short, unsigned short *,
		    unsigned short, unsigned short, const char *, ...);
void _MU_SAFE_DECL(mu_xvformat,
		   FILE *, unsigned short, unsigned short, unsigned short *,
		   unsigned short, unsigned short, const char *, va_list);
char *_MU_SAFE_DECL(mu_xvformat_string,
		    unsigned short, unsigned short, unsigned short *,
		    unsigned short, unsigned short, const char *, va_list);

#endif /* !_MU_SAFE_H */
